////////////////////////////////////////////////////////////////////////////////
// Copyright 2016 Prominic.NET, Inc.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
// http://www.apache.org/licenses/LICENSE-2.0 
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, 
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and 
// limitations under the License
// 
// Author: Prominic.NET, Inc.
// No warranty of merchantability or fitness of any kind. 
// Use this software at your own risk.
////////////////////////////////////////////////////////////////////////////////
package actionScripts.plugins.ant
{
	import flash.desktop.NativeProcess;
	import flash.desktop.NativeProcessStartupInfo;
	import flash.events.Event;
	import flash.events.NativeProcessExitEvent;
	import flash.events.ProgressEvent;
	import flash.filesystem.File;
	import flash.utils.IDataInput;
	
	import mx.controls.Alert;
	import mx.core.IFlexDisplayObject;
	
	import actionScripts.events.AddTabEvent;
	import actionScripts.events.GlobalEventDispatcher;
	import actionScripts.events.NewFileEvent;
	import actionScripts.events.RunANTScriptEvent;
	import actionScripts.factory.FileLocation;
	import actionScripts.locator.IDEModel;
	import actionScripts.plugin.IPlugin;
	import actionScripts.plugin.PluginBase;
	import actionScripts.plugin.actionscript.mxmlc.CommandLine;
	import actionScripts.plugin.settings.ISettingsProvider;
	import actionScripts.plugin.settings.event.SetSettingsEvent;
	import actionScripts.plugin.settings.vo.ISetting;
	import actionScripts.plugin.settings.vo.PathSetting;
	import actionScripts.ui.IContentWindow;
	import actionScripts.ui.editor.text.TextLineModel;
	import actionScripts.utils.HtmlFormatter;
	import actionScripts.utils.UtilsCore;
	import actionScripts.valueObjects.ConstantsCoreVO;
	import actionScripts.valueObjects.Settings;
	
	public class AntBuildPlugin extends PluginBase implements IPlugin, ISettingsProvider
	{
		public static const EVENT_ANTBUILD:String = "antbuildEvent";
		
		override public function get name():String			{ return "Ant Build Plugin"; }
		override public function get author():String		{ return "Moonshine Project Team"; }
		override public function get description():String	{ return "Ant Build Plugin. Esc exits."; }
		
		private var cmdFile:File;
		private var cmdLine:CommandLine;
		private var shellInfo:NativeProcessStartupInfo;
		private var nativeProcess:NativeProcess;
		private var errors:String = "";
		private var exiting:Boolean = false;
		private var _antHomePath:String;
		private var antPath:String = "bin/ant";
		private var _instance:AntBuildPlugin;
		private var workingDir:File;
		//private var antConfigureVo:AntConfigureVo;
		
		public function AntBuildPlugin() 
		{
			if (Settings.os == "win")
			{
				// in windows
				antPath+=".bat";
				cmdFile = new File("c:\\Windows\\System32\\cmd.exe");
			}
			else
			{
				// in mac
				//cmdFile = new File("/Applications/Utilities/Terminal.app");
				cmdFile = new File("/bin/bash");
			}
			
			GlobalEventDispatcher.getInstance().addEventListener(RunANTScriptEvent.ANT_BUILD,antBuildHandler);
			GlobalEventDispatcher.getInstance().addEventListener(NewFileEvent.EVENT_ANT_BIN_URL_SET, onAntURLSet);
			
			if (!_antHomePath && ConstantsCoreVO.IS_HELPER_DOWNLOADED_ANT_PRESENT) 
			{
				antHomePath = ConstantsCoreVO.IS_HELPER_DOWNLOADED_ANT_PRESENT.nativePath;
				
				var thisSettings: Vector.<ISetting> = getSettingsList();
				var pathSettingToDefaultSDK:PathSetting = thisSettings[0] as PathSetting;
				pathSettingToDefaultSDK.stringValue = antHomePath;
				dispatcher.dispatchEvent(new SetSettingsEvent(SetSettingsEvent.SAVE_SPECIFIC_PLUGIN_SETTING, null, "actionScripts.plugins.ant::AntBuildPlugin", thisSettings));
			}
		}
		
		public function get antHomePath():String
		{
			return _antHomePath;
		}
		public function set antHomePath(value:String):void
		{
			_antHomePath = value;
			if (_antHomePath == "")
			{
				model.antHomePath = null;
			}
			else
			{
				model.antHomePath = new FileLocation(value);
			}
		}
		
		override public function activate():void
		{
			super.activate();
			dispatcher.addEventListener(EVENT_ANTBUILD, antBuildHandler);
			cmdLine = new CommandLine();
			reset();
		}
		
		public function getSettingsList():Vector.<ISetting>
		{
			return Vector.<ISetting>([
				new PathSetting(this,'antHomePath', 'Ant Home', true, antHomePath)
			]);
		}
		
		override public function deactivate():void 
		{
			super.deactivate();
			nativeProcess.exit();
			reset();
			shellInfo = null;
			cmdLine = null;
		}
		
		private function reset():void 
		{
			nativeProcess = null;
		}
		
		private function onAntURLSet(event:NewFileEvent):void
		{
			antHomePath = event.filePath;
		}
		
		protected function antBuildHandler(event:Event):void 
		{
		  if(!model.defaultSDK || !model.antHomePath)
		  {
			/* Open AntBuild Screen in Popup
			antBuildPopup = new AntBuildPopup();
			PopUpManager.addPopUp(antBuildPopup, FlexGlobals.topLevelApplication as DisplayObject, false);
			PopUpManager.centerPopUp(antBuildPopup);*/
			  for each (var tab:IContentWindow in model.editors)
			  {
				  if (tab["className"] == "AntBuildScreen") 
				  {
					  model.activeEditor = tab;
					  return;
				  }
			  }
			  var antBuildScreen:IFlexDisplayObject = model.flexCore.getNewAntBuild();
			  antBuildScreen.addEventListener(AntBuildScreen.ANT_BUILD,antBuildSelected);
			  GlobalEventDispatcher.getInstance().dispatchEvent(
				  new AddTabEvent(antBuildScreen as IContentWindow)
			  );  
		  }
		  else
			  antBuildSelected(null);
			
		}
		private function antBuildSelected(e:Event):void{
			var SDKstr:String;
			if (!IDEModel.getInstance().antScriptFile)
			{
				Alert.show("You need to configure the script first by ANT -> Configure menu.");
				return;
			}
			else
				workingDir = IDEModel.getInstance().antScriptFile.fileBridge.getFile as File;
			
			var processArgs:Vector.<String> = new Vector.<String>;
			shellInfo = new NativeProcessStartupInfo();
			var antFile:FileLocation = model.antHomePath.resolvePath(antPath);
			var ANTstr:String = antFile.fileBridge.nativePath;
			ANTstr = UtilsCore.convertString(ANTstr);
			SDKstr =  model.defaultSDK.fileBridge.nativePath;
			if (Settings.os == "win")
			{
				/*processArgs.push("/C");
				processArgs.push("ant");
				processArgs.push("-file");
				processArgs.push(workingDir.nativePath);*/
				processArgs.push("/C");
				processArgs.push("set FLEX_HOME="+SDKstr+"&&"+ANTstr+" -file "+ workingDir.nativePath );
				shellInfo.arguments = processArgs;
				shellInfo.executable = cmdFile;
			}
			else 
			{
				processArgs.push("-c");
				processArgs.push("export FLEX_HOME="+SDKstr+"&&"+ANTstr+" -file "+ workingDir.nativePath );
				shellInfo.arguments = processArgs;
				shellInfo.executable = cmdFile;
				/*var command:String = "ant -file '"+ workingDir.nativePath + "'";
				shellInfo.workingDirectory = workingDir.parent;
				shellInfo.executable = File.documentsDirectory.resolvePath( "/usr/bin/osascript" );
				processArgs.push( "-e" );
				processArgs.push( 'tell application "Terminal" to activate do script "'+ command +'"'+'\n' );
				processArgs.push( 'end tell' );
				shellInfo.arguments = processArgs;*/
				/*processArgs.push("ant");
				processArgs.push("-file");
				processArgs.push(workingDir.nativePath);*/
			}
			shellInfo.workingDirectory = workingDir.parent; //pvo.folder;
			initShell();
		}
		private function initShell():void 
		{
			if (nativeProcess) {
				nativeProcess.exit();
				exiting = true;
				reset();
			} else {
				startShell();
			}
		}
		
		private function startShell():void 
		{
			nativeProcess = new NativeProcess();
			nativeProcess.addEventListener(ProgressEvent.STANDARD_OUTPUT_DATA, shellData);
			nativeProcess.addEventListener(ProgressEvent.STANDARD_ERROR_DATA, shellError);
			nativeProcess.addEventListener(NativeProcessExitEvent.EXIT, shellExit);
			nativeProcess.start(shellInfo);
			print("Ant build Running");
			flush();
		}
		
		private function shellData(e:ProgressEvent):void 
		{
			var output:IDataInput = nativeProcess.standardOutput;
			var data:String = output.readUTFBytes(output.bytesAvailable);
			
			var match:Array;
			
			match = data.match(/nativeProcess: Target \d not found/);
			if (match)
			{
				error("Target not found. Try again.");
				
			}
			
			match = data.match(/nativeProcess: Assigned (\d) as the compile target id/);
			
			if (data)
			{
				
				match = data.match(/(.*) \(\d+? bytes\)/);
				if (match) 
				{
					// Successful Build
					print("Done");
					
				}
			}
			if (data == "(nativeProcess) ") 
			{
				if (errors != "") 
				{
					compilerError(errors);
					errors = "";
				}
			}
			
			if (data.charAt(data.length-1) == "\n") data = data.substr(0, data.length-1);
			
			debug("%s", data);
			
		}
		private function shellError(e:ProgressEvent):void 
		{
			var output:IDataInput = nativeProcess.standardError;
			var data:String = output.readUTFBytes(output.bytesAvailable);
			
			var syntaxMatch:Array;
			var generalMatch:Array;
			var initMatch:Array;
			print(data);
			syntaxMatch = data.match(/(.*?)\((\d*)\): col: (\d*) Error: (.*).*/);
			if (syntaxMatch) {
				var pathStr:String = syntaxMatch[1];
				var lineNum:int = syntaxMatch[2];
				var colNum:int = syntaxMatch[3];
				var errorStr:String = syntaxMatch[4];
				pathStr = pathStr.substr(pathStr.lastIndexOf("/")+1);
				errors += HtmlFormatter.sprintf("%s<weak>:</weak>%s \t %s\n",
					pathStr, lineNum, errorStr); 
			}
			
			generalMatch = data.match(/(.*?): Error: (.*).*/);
			if (!syntaxMatch && generalMatch)
			{ 
				pathStr = generalMatch[1];
				errorStr  = generalMatch[2];
				pathStr = pathStr.substr(pathStr.lastIndexOf("/")+1);
				
				errors += HtmlFormatter.sprintf("%s: %s", pathStr, errorStr);
			}
			
			debug("%s", data);
		}
		
		private function shellExit(e:NativeProcessExitEvent):void 
		{
			debug("FSCH exit code: %s", e.exitCode);
			if (exiting) {
				exiting = false;
				startShell();
			}
			reset();
			
		}
		private function flush():void 
		{
			
			
		}
		protected function compilerError(...msg):void 
		{
			var text:String = msg.join(" ");
			var textLines:Array = text.split("\n");
			var lines:Vector.<TextLineModel> = Vector.<TextLineModel>([]);
			for (var i:int = 0; i < textLines.length; i++)
			{
				if (textLines[i] == "") continue;
				text = "<error> ⚡  </error>" + textLines[i]; 
				var lineModel:TextLineModel = new TextLineModel(text);
				lines.push(lineModel);
			}
			outputMsg(lines);
		}
		
	}
}