////////////////////////////////////////////////////////////////////////////////
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
// http://www.apache.org/licenses/LICENSE-2.0 
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, 
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and 
// limitations under the License
// 
// No warranty of merchantability or fitness of any kind. 
// Use this software at your own risk.
// 
////////////////////////////////////////////////////////////////////////////////
package actionScripts.plugins.balsamiq
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.filesystem.*;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.utils.ByteArray;
	import mx.collections.CursorBookmark;
	import mx.controls.Alert;
	import flash.xml.XMLNode;
	import flash.xml.XMLNodeType;	
	
	public class BalsamiqToFlexJSConvert extends Sprite
	{
		private var xmlData:XML;
		private var arrControls:Array;
		private var arrComponents:Array;
		private var strMXMLData:String;
		private var strXMLPath:String;
		private var strMXMLExportFilePath:String;
		private var XVarianceAllowed:int;
		private var YVarianceAllowed:int;
		private var Debugging:Boolean;
		private var blnValidXML:Boolean;
		public var callBackF:Function;
		
		public function isValidXML():Boolean
		{
			return blnValidXML;
		}
		
		public function BalsamiqToFlexJSConvert(XMLPath:String, MXMLExportFilePath:String):void
		{
			try{
			strXMLPath = XMLPath;
			strMXMLExportFilePath = MXMLExportFilePath;
			blnValidXML = true;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		public function loadXML(mDebugging:Boolean):void
		{
			try{
			var myXML:XML = new XML();
			var XML_URL:String = strXMLPath;
			var myXMLURL:URLRequest = new URLRequest(XML_URL);
			var myLoader:URLLoader = new URLLoader(myXMLURL);
			
			Debugging = mDebugging;
			myLoader.addEventListener(Event.COMPLETE, xmlDownloaded);
			}
			catch (e:Error)
			{
				Alert.show("Could not open the XML file for processing:  \"" + strXMLPath + "\"");
				blnValidXML = false;
				trace(e.getStackTrace());
			}
		}

		public function validateXML(callback:Function):void
		{
			try{
			var XML_URL:String = strXMLPath;
			var myXMLURL:URLRequest = new URLRequest(XML_URL);
			var myLoader:URLLoader = new URLLoader(myXMLURL);
			
			callBackF = callback;
			
			myLoader.addEventListener(Event.COMPLETE, xmlValidation);
			}
			catch (e:Error)
			{
				Alert.show("Could not open the XML file for validation:  \"" + strXMLPath + "\"");
				blnValidXML = false;
				trace(e.getStackTrace());
			}
		}


		private function xmlValidation(event:Event):void
		{
			try{
				xmlData = new XML(event.target.data);
				callBackF();
			}
			catch (e:Error)
			{
				Alert.show("The XML file \"" + strXMLPath + "\" could not be parsed.");
				blnValidXML = false;
				callBackF();
				trace(e.getStackTrace());
			}
		}

		
		private function xmlDownloaded(event:Event):void
		{
			try{
				xmlData = new XML(event.target.data);
				
				XVarianceAllowed = 10;
				YVarianceAllowed = 10;
				
				extractControls();
				exportToFile();
			}
			catch (e:Error)
			{
				Alert.show("The XML file \"" + strXMLPath + "\" could not be parsed during processing.");
				blnValidXML = false;
				trace(e.getStackTrace());
			}
		}
		
		
		private function collectMXMLData():void
		{
			try{
			
			var intLen:int = arrComponents.length;
			var intKtr:int;
			
			if(intLen>0)
				strMXMLData = "";
			
			for (intKtr = 0; intKtr < intLen; intKtr++)
			{
				if (arrComponents[intKtr].controlTypeID == "custom")
					strMXMLData = strMXMLData + arrComponents[intKtr].text;
			}
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		public function exportToFile():void
		{
			try{
			var file:File = File.desktopDirectory.resolvePath(strMXMLExportFilePath);
			var stream:FileStream = new FileStream();
			var strMXMLAll:String;
			var xml:XML;
			var strWrite:String;
			
			collectMXMLData();
			
			stream.open(file, FileMode.WRITE);
			
			strMXMLAll = "<?xml version=\"1.0\" encoding=\"utf-8\"?><js:Application xmlns:fx=\"http://ns.adobe.com/mxml/2009\" xmlns:js=\"library://ns.apache.org/flexjs/basic\"  xmlns:local=\"*\" xmlns:ns=\"library://ns.apache.org/flexjs/html5\"><fx:Declarations></fx:Declarations><fx:Script><![CDATA[]]></fx:Script><fx:Style>.alignTop{vertical-align: top;}</fx:Style><js:valuesImpl><js:SimpleCSSValuesImpl /></js:valuesImpl><js:initialView><js:ViewBase><js:Container width=\"100%\" height=\"100%\"  style=\"border:1px solid red\">";

			strMXMLAll = strMXMLAll + "<js:beads><js:VerticalLayout />	</js:beads>";
			
			strMXMLAll = strMXMLAll + strMXMLData;
			
			strMXMLAll = strMXMLAll + "\n</js:Container> \n </js:ViewBase> \n </js:initialView> \n </js:Application>";			
			
			xml = new XML(strMXMLAll);

			strWrite = xml.toXMLString();
			//strWrite = strMXMLAll;
			
			
			strWrite = strReplace(strWrite, "  ", String.fromCharCode(9));
			
			stream.writeUTFBytes(strWrite);
			
			stream.close();
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		public function getMXMLCode():String
		{
			try{
			var strMXMLAll:String;
			var xml:XML;
			
			strMXMLAll = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<js:Application xmlns:fx=\"http://ns.adobe.com/mxml/2009\" xmlns:js=\"library://ns.apache.org/flexjs/basic\" \n xmlns:local=\"*\" xmlns:ns=\"library://ns.apache.org/flexjs/html5\">\n\n<fx:Declarations></fx:Declarations>\n\n<fx:Script><![CDATA[]]></fx:Script>\n\n<js:valuesImpl><js:SimpleCSSValuesImpl />\n\n</js:valuesImpl>\n\n<js:initialView>\n\n<js:ViewBase>\n<js:Container width=\"100%\" height=\"100%\"  style=\"border:1px solid red\">";
			
			strMXMLAll = strMXMLAll + "<js:beads><js:VerticalLayout />	</js:beads>";
			
			strMXMLAll = strMXMLAll + strMXMLData;
			
			strMXMLAll = strMXMLAll + "\n</js:Container> \n </js:ViewBase> \n </js:initialView> \n </js:Application>";
			
			xml = new XML(strMXMLAll);
			
			var strWrite:String = xml.toXMLString();
			
			strWrite = strReplace(strWrite, "  ", String.fromCharCode(9));
			
			return strWrite;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return "";
		}
		
		private function strReplace(str:String, search:String, replace:String):String
		{
			return str.split(search).join(replace);
		}
		
		public function extractControls():void
		{
			try{
			var intParentContainerIndex:int;
			
			intParentContainerIndex = -1;
			extractAllControls();
			resetXCoordinates();
			resetYCoordinates();
			resetAXAYCoordinates();
			//resetXHYHCoordinates();
			
			convertControlsInContainer(intParentContainerIndex);
			intParentContainerIndex = getInnerMostContainer();
			
			do
			{
				//convertControlsInContainer(intParentContainerIndex);
				assembleControlsInContainer(intParentContainerIndex);
				intParentContainerIndex = getInnerMostContainer();
				
					//break;
			} while (intParentContainerIndex != -1);
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		public function getInnerMostContainer():int
		{
			try{
			var intLen:int = 0;
			var intKtr:int = 0;
			var intPos:int = -1;
			var intLastContainerX:int;
			var intLastContainerY:int;
			
			//arrControls.sortOn("XY", Array.NUMERIC);
			
			intLen = arrComponents.length;
			
			for (intKtr = 0; intKtr < intLen; intKtr++)
			{
				var intX:Number = 0;
				var intY:Number = 0;
				var blnContainerControl:Boolean = false;
				
				intX = arrComponents[intKtr].x;
				intY = arrComponents[intKtr].x;
				
				blnContainerControl = isContainerControl(arrComponents[intKtr].controlTypeID);
				
				if (blnContainerControl)
				{
					if (intX >= intLastContainerX && intY >= intLastContainerY)
					{
						intPos = intKtr;
						intLastContainerX = intX;
						intLastContainerY = intY;
					}
				}
				
			}
			
			return intPos;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return -1;
		}
		
		public function isContainerControl(strControlName:String):Boolean
		{
			try{
			switch (strControlName)
			{
			case "com.balsamiq.mockups::BrowserWindow": 
				return true;
			case "com.balsamiq.mockups::Canvas": 
				return true;
			case "com.balsamiq.mockups::FieldSet": 
				return true;
			case "com.balsamiq.mockups::TitleWindow": 
				return true;
//			case "__group__": 
//				return true;
			}
			return false;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return false;
		}
		
		public function isChildOfContainer(Parent:Object, Child:Object):Boolean
		{
			try{
				var intParentx:int;
				var intParenty:int;
				var intParentHeight:int;
				var intParentWidth:int;
				var intChildx:int;
				var intChildy:int;
				var intChildHeight:int;
				var intChildWidth:int;
				var blnResult:Boolean = true;
				var strParentID:String = "";
				var strChildGroupID:String = "";
				var blnChildContainerControl:Boolean = false;
							
				blnChildContainerControl = isContainerControl(Child.controlTypeID.toString());
				
				
				
				if (Parent != null)
				{
					intParentx = parseInt(Parent.x);
					intParenty = parseInt(Parent.y);
					intParentWidth = parseInt(Parent.w);
					intParentHeight = parseInt(Parent.h);
					intChildx = parseInt(Child.x);
					intChildy = parseInt(Child.y);1
					intChildWidth = parseInt(Child.w);
					intChildHeight = parseInt(Child.h);
					strChildGroupID = Child.isInGroup.toString();
					strParentID = Parent.controlID.toString();
					
					if (intParentx <= intChildx && intParenty <= intChildy)
					{
						if(! blnChildContainerControl || (intParentx + intParentWidth >= intChildx + intChildWidth && intParenty + intParentHeight >= intChildy + intChildHeight))
						{
							blnResult = true;
						}
						else
						{
							blnResult = false;
						}
							
					}
					else
						blnResult = false;
				}
				
				return blnResult;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return false;
		}
		
		public function extractAllControls():void
		{
			try{
			arrControls = new Array();
			
			for each (var object:XML in xmlData.controls..*)
			{
				if (object.name() == "control")
				{
					var strText:String = "";
					var intX:Number = 0;
					var intY:Number = 0;
					var intXY:Number = 0;
					var strSize:String = "";
					var strColor:String = "";
					var strAlign:String = "";
					var strIsInGroup:String = "";
					var strLocked:String = "";
					
					
					intX = parseInt(object.@x);
					intY = parseInt(object.@y);
					intXY = intY * 1000 + intX;
					
					strText = object.controlProperties.text.toString();
					strSize = object.controlProperties.size.toString();
					strColor = object.controlProperties.color.toString();
					strAlign = object.controlProperties.align.toString();
					strIsInGroup = object.@isInGroup.toString();
					strLocked = object.@locked.toString();
					
					arrControls.push({locked: strLocked, isInGroup: strIsInGroup, size: strSize, color: strColor, align: strAlign, text: strText, controlID: object.@controlID.toString(), controlTypeID: object.@controlTypeID.toString(), x: object.@x.toString(), y: object.@y.toString(), w: object.@w.toString(), h: object.@h.toString(), measuredW: object.@measuredW.toString(), measuredH: object.@measuredH.toString(), XY: intXY});
				}
			}
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		public function convertControlsInContainer(intParentContainerIndex:int):void
		{
			try{
			var intLen:int = 0;
			var intKtr:int = 0;
			
			arrControls.sortOn("AXAY", Array.NUMERIC);
			
			arrComponents = new Array();
			
			intLen = arrControls.length;
			
			for (intKtr = 0; intKtr < intLen; intKtr++)
			{
				var strComponentName:String = "";
				var strCSSStyle:String = "";
				
				strComponentName = componentMapper(arrControls[intKtr].controlTypeID);
				
				if (strComponentName != "")
				{
					var strPropertyName:String = "";
					var strFJXML:String = "";
					var intWidth:int = 0;
					var intHeight:int = 0;
					var strPrepareDataProvider:String = "";
					var strID:String = "";
					
					if(intParentContainerIndex>=0)
						strID = intParentContainerIndex.toString() + "_" + intKtr.toString();
					else
						strID = intKtr.toString();
					
						
					strFJXML = "<" + strComponentName;
					
					strPropertyName = propertyMapper(arrControls[intKtr].controlID);
					strFJXML = strFJXML + " id=\"id" + strID + "\"";
					
					strPropertyName = propertyMapper(arrControls[intKtr].text);
					if (strPropertyName != "")
					{
						if (strComponentName == "js:ComboBox" || strComponentName == "js:DropDownList" || strComponentName == "js:ButtonBar" || strComponentName == "js:List")
							strPrepareDataProvider = prepareDataProvider(strPropertyName);
						else if (strComponentName == "js:NumericStepper")
							strFJXML = strFJXML + " maximum=\"100\" minimum=\"1\" stepSize=\"1\" value=\"" + strPropertyName + "\"";
						else if (strComponentName == "js:HRule")
							""
						else
							strFJXML = strFJXML + " text=\"" + htmlEscape(strPropertyName) + "\"";
					}

					intWidth = getControlWidth(arrControls[intKtr]);

					if (strComponentName == "js:ButtonBar")
						intWidth = intWidth * 1.8;
						
					strFJXML = strFJXML + " width=\"" + intWidth + "\"";
					
					intHeight = getControlHeight(arrControls[intKtr]);
					
					if (strComponentName == "js:HRule")
						intHeight = intHeight/10;
					
					strCSSStyle = getCSSStyle(arrControls[intKtr]);
					
					strFJXML = strFJXML + " height=\"" + intHeight + "\"" + strCSSStyle + " className=\"alignTop\">";
					
					strFJXML = strFJXML + strPrepareDataProvider;

					strFJXML = strFJXML + "</" + strComponentName + ">";
					
					strFJXML = unescape(strFJXML);
					
					arrComponents.push({text: strFJXML, controlTypeID: arrControls[intKtr].controlTypeID, x: arrControls[intKtr].x, y: arrControls[intKtr].y, XY: arrControls[intKtr].XY, w: intWidth, h: intHeight, isInGroup: arrControls[intKtr].isInGroup, locked: arrControls[intKtr].locked, controlID: arrControls[intKtr].controlID, size: arrControls[intKtr].size, color: arrControls[intKtr].color, align: arrControls[intKtr].align});
				}
			}
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		private function getControlWidth(ObjControl:Object):int
		{
			try{
			var intWidth:int = 0;
			var intW:int = 0;
			var intMeasuredW:int = 0;
			var blnContainerControl:Boolean = false;
			var strControlName:String = ObjControl.controlTypeID.toString();
			
			blnContainerControl = isContainerControl(strControlName);
			
			intW = parseInt(ObjControl.w);
			intMeasuredW = parseInt(ObjControl.measuredW);

			if (ObjControl.w != "-1")
			{
				// Here we need to calculate the width based on the control type. 
				// For some controls, we need to take min width
				// and for container controls we need to take max width
				var intMinWidth:int = 0;
				var intMaxWidth:int = 0;
				
				if (intW > intMeasuredW)
				{
					intMinWidth = intMeasuredW;
					intMaxWidth = intW;
				}
				else
				{
					intMinWidth = intW;
					intMaxWidth = intMeasuredW;
				}
				
				if(blnContainerControl ||  strControlName == "com.balsamiq.mockups::HRule" || strControlName == "com.balsamiq.mockups::ComboBox")
					intWidth = intMaxWidth;
				else
					intWidth = intMinWidth;
			}
			else
				intWidth = intMeasuredW;
			
			return intWidth;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return -1;
		}

		private function getControlHeight(ObjControl:Object):int
		{
			try{
			var intHeight:int = 0;
			var intH:int = 0;
			var intMeasuredH:int = 0;
			var blnContainerControl:Boolean = false;
			
			blnContainerControl = isContainerControl(ObjControl.controlTypeID.toString());
			
			trace("blnContainerControl : " + blnContainerControl);
			
			intH = parseInt(ObjControl.h);
			intMeasuredH = parseInt(ObjControl.measuredH);
			
			if(intH==0)
				intH = intMeasuredH;
			
			if(intMeasuredH==0)
				intMeasuredH = intH;
			

			trace("intH : " + intH );
			trace("intMeasuredH : " + intMeasuredH );

			if (intH>0)
			{
				// Here we need to calculate the Height based on the control type. 
				// For some controls, we need to take min Height
				// and for container controls we need to take max Height
				var intMinHeight:int = 0;
				var intMaxHeight:int = 0;
				
				if (intH > intMeasuredH)
				{
					intMinHeight = intMeasuredH;
					intMaxHeight = intH;
				}
				else
				{
					intMinHeight = intH;
					intMaxHeight = intMeasuredH;
				}
				
				if(blnContainerControl)
					intHeight = intMaxHeight;
				else
					intHeight = intMinHeight;
			}
			else
				intHeight = intMeasuredH;

			trace("intHeight : " + intHeight );

			
			return intHeight;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return -1;
		}

		public function htmlEscape(str:String):String {
			try{
			var strReturn:String = "";
			
			strReturn = XML( new XMLNode( XMLNodeType.TEXT_NODE, str ) ).toXMLString();
			
			strReturn = strReturn.split("\"").join ("&quot;");
			
			strReturn = strReturn.replace(/</gi, "&lt;");
			strReturn = strReturn.replace(/>/gi, "&gt;");
	
			
			return strReturn;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return "";
		}
		
		private function getCSSStyle(objControl:Object):String
		{
			try{
			var strStyle:String = "";
			
			if (objControl.color != "")
				strStyle = strStyle + "color:'" + objControl.color + "';";

			if (objControl.size != "")
				strStyle = strStyle + "fontSize:" + objControl.size + ";";

			if (objControl.align != "")
				strStyle = strStyle + "textAlign:'" + objControl.align + "';";


			if (strStyle != "")
			{
				strStyle = strStyle.slice(0, -1);
				
				strStyle = " style=\"" + strStyle + "\" "
			}
			
			return strStyle;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return "";
		}
		
		public function prepareDataProvider(strData:String):String
		{
			try{
			var strDataProvider:String;
			var arrData:Array;
			
			var intLen:int = 0;
			var intKtr:int = 0;
			
			if (strData.indexOf(",") >= 0)
				arrData = strData.split(",");
			else if (strData.indexOf(String.fromCharCode(13)) >= 0)
				arrData = arrData.split(String.fromCharCode(13));
			else
				arrData = strData.split(String.fromCharCode(10));
			
			intLen = arrData.length;
			
			strDataProvider = "<js:dataProvider><fx:Array>";
			
			for (intKtr = 0; intKtr < intLen; intKtr++)
			{
				strDataProvider = strDataProvider + "<fx:String>" + htmlEscape(arrData[intKtr]) + "</fx:String>";
			}
			
			strDataProvider = strDataProvider + "</fx:Array></js:dataProvider>";
			
			return strDataProvider;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return "";
		}
		
		public function resetXCoordinates():void
		{
			try{
				var intLen:int = 0;
				var intKtr:int = 0;
				var intLastX:int;
				var intLastAX:int;
				
				
				intLen = arrControls.length;

				if (intLen > 0)
				{
				arrControls.sortOn("x", Array.NUMERIC);

				intLastX = arrControls[0].x;
				
				for (intKtr = 1; intKtr < intLen; intKtr++)
				{
					arrControls[intKtr].ax = arrControls[intKtr].x;
					
					if (arrControls[intKtr].x - intLastAX < XVarianceAllowed)
					{
						arrControls[intKtr].ax = intLastAX;
					}
					
					intLastX = arrControls[intKtr].x
					intLastAX = arrControls[intKtr].ax
				}
					
				}
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		public function resetYCoordinates():void
		{
			try{
			var intLen:int = 0;
			var intKtr:int = 0;
			var intLastY:int;
			var intLastAY:int;
			
			intLen = arrControls.length;

			if (intLen > 0)
			{
			arrControls.sortOn("y", Array.NUMERIC);
			

			intLastY = arrControls[0].y;
			

			for (intKtr = 1; intKtr < intLen; intKtr++)
			{
				arrControls[intKtr].ay = arrControls[intKtr].y;
				if (arrControls[intKtr].y - intLastAY < YVarianceAllowed)
				{
					arrControls[intKtr].ay = intLastAY;
				}
				
				intLastY = arrControls[intKtr].y;
				intLastAY = arrControls[intKtr].ay;
			}
			}
			
		
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		public function resetXYCoordinates():void
		{
			try{
			var intLen:int = 0;
			var intKtr:int = 0;
			
			intLen = arrControls.length;
			
			for (intKtr = 1; intKtr < intLen; intKtr++)
			{
				arrControls[intKtr].XY = Number(arrControls[intKtr].y) * 1000 + Number(arrControls[intKtr].x);
			}
			
			arrControls.sortOn("XY", Array.NUMERIC);
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		public function resetAXAYCoordinates():void
		{
			try{
			var intLen:int = 0;
			var intKtr:int = 0;
			
			intLen = arrControls.length;
			
			for (intKtr = 1; intKtr < intLen; intKtr++)
			{
				arrControls[intKtr].AXAY = Number(arrControls[intKtr].ay) * 1000 + Number(arrControls[intKtr].ax);
			}
			
			arrControls.sortOn("AXAY", Array.NUMERIC);
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		

		public function resetXHYHCoordinates():void
		{
			try{
			var intLen:int = 0;
			var intKtr:int = 0;
			
			intLen = arrControls.length;
			
			for (intKtr = 0; intKtr < intLen; intKtr++)
			{
				var intWidth:int = 0;
				var intHeight:int = 0;
				
				if (!isContainerControl(arrControls[intKtr].controlTypeID.toString()))
				{
					if (arrControls[intKtr].w != "-1")
						intWidth = Number(arrControls[intKtr].w);
					else
						intWidth = Number(arrControls[intKtr].measuredW);
					
					if (arrControls[intKtr].h != "-1")
						intHeight = arrControls[intKtr].h;
					else
						intHeight = arrControls[intKtr].measuredH;
				}
				
				arrControls[intKtr].XHYH = (Number(arrControls[intKtr].y) + intHeight) * 1000 + (Number(arrControls[intKtr].x) + intWidth);
			}
			
			arrControls.sortOn("XHYH", Array.NUMERIC);
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		public function assembleControlsInContainer(intParentContainerIndex:int):void
		{
			try{
			var intLen:int = 0;
			var intKtr:int = 0;
			var strParentContainerHeader:String = "";
			var strParentContainerFooter:String = "";
			var strContainerHeader:String = "";
			var strContainerFooter:String = "";
			var intLastY:int = 0;
			var intLastX:int = 0;
			var intLastW:int = 0;
			var intLastH:int = 0;
			var intLastMaxH:int = 0;
			var intLastMaxY:int = 0;
			var intContainerX:int = 0;
			var intContainerY:int = 0;
			var blnFirstControl:Boolean = true;
			var intHCount:int=0;
			var intMarginH:int = 0;
			
			strContainerHeader = "<js:Container style=\"border:0px solid black\"><js:beads><js:HorizontalLayout /></js:beads>"
			if (intParentContainerIndex >= 0)
			{
				var strCSSStyle:String = getCSSStyle(arrControls[intKtr]);
				// Need to append strCSSStyle in ParentContainerHeader. The style attributes in this are being lost otherwise
				strParentContainerHeader = "<js:Container height=\"" + arrComponents[intParentContainerIndex].h + "\" width=\"" + arrComponents[intParentContainerIndex].w + "\" style=\"border:1px solid black\"><js:beads><js:VerticalLayout /></js:beads>"
				strParentContainerFooter = "</js:Container>"
			}
			strContainerFooter = "</js:Container>"
			
			if (intParentContainerIndex >= 0)
			{
				intContainerX = arrComponents[intParentContainerIndex].x;
				intContainerY = arrComponents[intParentContainerIndex].y;
			}
			
			intLen = arrComponents.length;
			
			strMXMLData = "";
			
			for (intKtr = 0; intKtr < intLen; intKtr++)
			{
				var intSpacerW:int = 0;
				var intSpacerH:int = 0;
				var intInlineBlock:int = 4;  	// To adjust the extra space due to Display:Inline-Block;
				var strSize:String = "";
				var strColor:String = "";
				var strAlign:String = "";
				
				if (intParentContainerIndex != intKtr)
				{
					var intDiff:int = arrComponents[intKtr].y - (intLastY + intContainerY);
					var isChild:Boolean = isChildOfContainer(arrComponents[intParentContainerIndex], arrComponents[intKtr]);
					
					strSize = arrComponents[intKtr].size;
					strColor = arrComponents[intKtr].color;
					strAlign = arrComponents[intKtr].align;
					
					if (isChild)
					{
						if (intDiff < 0)
							intDiff = intDiff * -1;
						
						if (blnFirstControl)
						{
							intHCount = intHCount + 1;
							intSpacerH = arrComponents[intKtr].y - intLastY - intContainerY - intInlineBlock;
							strMXMLData = strMXMLData + strParentContainerHeader ;
							if (intSpacerH > 0)
								strMXMLData = strMXMLData + "\n" + "<js:Spacer style=\"border:0px solid blue\" height=\"" + intSpacerH + "\" width=\"100%\"/>";
							
							strMXMLData = strMXMLData + strContainerHeader;
							
							blnFirstControl = false;
						}
						else if ( (intLastY + intLastH) <= (arrComponents[intKtr].y - intContainerY) || intLastX >= arrComponents[intKtr].x)
						{
							intLastH = 0;
							intLastW = 0;
							intLastX = 0;
							intSpacerH = arrComponents[intKtr].y - intLastY - intLastMaxH - intContainerY - intInlineBlock;
							
							if (intSpacerH > 0)
								strMXMLData = strMXMLData + "\n" + "<js:Spacer style=\"border:0px solid blue\" height=\"" + intSpacerH + "\" width=\"100%\"/>";
							
							intLastMaxH = 0;
							intLastMaxY = 0;
							intMarginH = 0;
							strMXMLData = strMXMLData + strContainerFooter;
							strMXMLData = strMXMLData + strContainerHeader;
						}
						else
						{
							// Instead of Spacer, need to leave margin here.
							intHCount = intHCount + 1;
							intMarginH = intMarginH + parseInt(arrComponents[intKtr].y) - intLastY - intContainerY;

							if (intMarginH > 0)
								applyMargin(intKtr, intMarginH, 0, 0, 0);
							
							//	strMXMLData = strMXMLData + "\n" + "<js:Spacer style=\"border:0px solid blue\" height=\"" + intSpacerH + "\" width=\"100%\"/>";
						}
						
						intSpacerW = arrComponents[intKtr].x - intLastX - intLastW - intContainerX;
						
						if (intSpacerW > 0)
							strMXMLData = strMXMLData + "\n" + "<js:Spacer style=\"border:0px solid yellow\" width=\"" + intSpacerW + "\" height=\"1\"/>"
						
						strMXMLData = strMXMLData + "\n" + arrComponents[intKtr].text;
						intLastY = arrComponents[intKtr].y - intContainerY;
						intLastX = arrComponents[intKtr].x - intContainerX;
						intLastW = arrComponents[intKtr].w;
						intLastH = arrComponents[intKtr].h;
						
						if (arrComponents[intKtr].y > intLastMaxY)
							intLastMaxY = arrComponents[intKtr].y;
						
						if (arrComponents[intKtr].h > intLastMaxH)
							intLastMaxH = arrComponents[intKtr].h;
						
						arrComponents[intKtr] = "";
					}
				}
				
			}
			
			if(blnFirstControl==false)
				strMXMLData = strMXMLData + strContainerFooter + strParentContainerFooter;
			
			if (intParentContainerIndex >= 0)
			{
				arrComponents[intParentContainerIndex].controlTypeID = "custom";
				arrComponents[intParentContainerIndex].text = strMXMLData;
			}
			
			arrComponents = trimArray(arrComponents);
			
			
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		private function applyMargin(intPos:int, intMarginT:int, intMarginR:int, intMarginB:int, intMarginL:int):void
		{
			try{
			var strMarginStyle:String = "";
			var strHTML:String = "";
			var strHTMLNew:String = "";
			var intStylePos:int = -1;			
			
			strHTML = arrComponents[intPos].text;
			intStylePos = strHTML.indexOf("style=");
			
			if (intStylePos == -1)
			{
				strMarginStyle = "margin:" + intMarginT.toString() + "px " + intMarginR.toString() + "px " + intMarginB.toString() + "px " + intMarginL.toString() + "px"
				intStylePos = strHTML.indexOf(">");
				strHTMLNew = strHTML.slice(0, intStylePos) + " style=\"" + strMarginStyle + "\"" + strHTML.slice(intStylePos);
			}
			else
			{
				strMarginStyle = "margin:" + intMarginT.toString() + "px " + intMarginR.toString() + "px " + intMarginB.toString() + "px " + intMarginL.toString() + "px;"
				strHTMLNew = strHTML.slice(0, intStylePos+7) + strMarginStyle + strHTML.slice(intStylePos+7);
			}
			
			arrComponents[intPos].text = strHTMLNew;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
		}
		
		private function trimArray(arrToTrim:Array):Array
		{
			try{
			var intLen:int = 0;
			var intKtr:int = 0;
			
			var arrTrimmed:Array = new Array();
			
			intLen = arrToTrim.length;
			
			for (intKtr = 0; intKtr < intLen; intKtr++)
			{
				if (arrToTrim[intKtr] != "")
				{
					arrTrimmed.push(arrToTrim[intKtr]);
				}
			}
			
			return arrTrimmed;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return null;
		}
		
		private function propertyMapper(strName:String):String
		{
			return unescape(strName);
		}
		
		private function componentMapper(strName:String):String
		{
			try{
			switch (strName)
			{
			case "com.balsamiq.mockups::DateChooser": 
				return "js:TextInput";
				break;
			
			case "com.balsamiq.mockups::Image": 
				return "js:Image";
				break;
			
			case "com.balsamiq.mockups::BarChart": 
				return "js:BarChart";
				break;
			
			case "com.balsamiq.mockups::ColumnChart": 
				return "js:ColumnChart";
				break;
			
			case "com.balsamiq.mockups::LineChart": 
				return "js:LineChart";
				break;
			
			case "com.balsamiq.mockups::PieChart": 
				return "js:PieChart";
				break;
			
			case "com.balsamiq.mockups::TabBar": 
				return "js:TabBar";
				break;
			
			case "com.balsamiq.mockups::ButtonBar": 
				return "js:ButtonBar";
				break;
			
			case "com.balsamiq.mockups::DropDownList": 
				return "js:DropDownList";
				break;
			
			case "com.balsamiq.mockups::ComboBox": 
				return "js:DropDownList";
				break;
			
			case "com.balsamiq.mockups::CheckBox": 
				return "js:CheckBox";
				break;
	
			case "com.balsamiq.mockups::List": 
				return "js:List";
				break;
			
			case "com.balsamiq.mockups::HRule": 
				return "js:HRule";
				break;
			
			case "com.balsamiq.mockups::VRule": 
				return "js:VRule";
				break;
			
			case "com.balsamiq.mockups::NumericStepper": 
				return "js:NumericStepper";
				break;
			
			case "com.balsamiq.mockups::RadioButton": 
				return "js:RadioButton";
				break;
			
			case "com.balsamiq.mockups::TextArea": 
				return "js:TextArea";
				break;
			
			case "com.balsamiq.mockups::TextInput": 
				return "js:TextInput";
				break;
			
			case "com.balsamiq.mockups::Button": 
				return "js:TextButton";
				break;
			
			case "com.balsamiq.mockups::Label": 
				return "js:Label";
				break;
			
			case "com.balsamiq.mockups::Title": 
				return "js:Label";
				break;
			
			case "com.balsamiq.mockups::Paragraph": 
				return "js:Label";
				break;

			case "com.balsamiq.mockups::BrowserWindow": 
				return "js:Container";
				break;
			
			case "com.balsamiq.mockups::TitleWindow": 
				return "js:Container";
				break;
			
			case "com.balsamiq.mockups::TabBar": 
				return "js:Container";
				break;
			
			case "com.balsamiq.mockups::FieldSet": 
				return "js:Container";
				break;
			
			case "com.balsamiq.mockups::Canvas": 
				return "js:Container";
				break;

//			case "__group__": 
//				return "js:Container";
//				break;
			}
			
			}
			catch (e:Error)
			{
				trace(e.getStackTrace());
			}
			return "";

		}
	}
}

